/*
 * PAD handling module.
 */

#include "../config.h"
#include "pad.h"
#include "../psx/gpu.h"
#include "../psx/interpreter.h"
#include "../psx/memory.h"
#include "../psx/msg.h"

#include <dlfcn.h>
#include <stdlib.h>

static void* so = NULL;

static int (*psx_pad_init)(int);
static void (*psx_pad_shutdown)(void);
static int (*psx_pad_open)(unsigned long*);
static int (*psx_pad_close)(void);
static unsigned char (*psx_pad_start_poll)(int);
static unsigned char (*psx_pad_poll)(unsigned char);

static uint32_t buffer_addr = 0;

static void load_plugin(void)
{
    if (so)
        return; /* Already loaded */

    so = dlopen(LIBDIR "/libpad.so", RTLD_NOW);
    if (!so)
        psx_fail("No PAD plugin installed on this system !\n");

#define LOAD_(sym, sosym) \
    psx_pad_##sym = dlsym(so, "PAD" #sosym); \
    if (!psx_pad_##sym) \
        psx_fail("Unable to find symbol %s in the PAD plugin !\n", #sosym)

#define LOAD(sym) LOAD_(sym, sym)

    LOAD(init);
    LOAD(shutdown);
    LOAD(open);
    LOAD(close);
    LOAD(poll);
    LOAD_(start_poll, startPoll);

    psx_pad_init(1);
    psx_pad_open(psx_gpu_get_display());
}

void psx_bios_pad_init(uint32_t* GPR)
{
    uint32_t old = psx_mem_read_32(0x1F801074);

    buffer_addr = GPR[A1];

    psx_mem_write_32(0x1F801074, old | 1);
    psx_mem_write_32(buffer_addr, 0xFFFFFFFF);

    load_plugin();
}

void psx_bios_pad_stop(uint32_t* GPR)
{
    buffer_addr = 0;
}

void psx_bios_pad_interrupt(void)
{
    if (buffer_addr)
    {
        uint32_t* p = (uint32_t*)psx_mem_ram_ptr(buffer_addr & (RAM_SIZE - 1));

        /* XXX shamefully copied from PCSX-R, will try to understand that
         * after our first oral */
        psx_pad_start_poll(1);
        if (psx_pad_poll(0x42) == 0x23)
        {
            psx_pad_poll(0);
            *p = psx_pad_poll(0) << 8;
            *p |= psx_pad_poll(0);
            psx_pad_poll(0);
            *p &= ~((psx_pad_poll(0) > 0x20) ? 1 << 6 : 0);
            *p &= ~((psx_pad_poll(0) > 0x20) ? 1 << 7 : 0);
        }
        else
        {
            psx_pad_poll(0);
            *p = psx_pad_poll(0) << 8;
            *p |= psx_pad_poll(0);
        }
    }
}
