module ComputerPsxFile = Psx_file.Make (Os_fs)

type cfg = {
    mutable fname : string ;
    mutable act : Psx_file.t -> unit
}

let dump_header f =
    let module P = Psx_file in
    let header = f.P.header in
    let pi32 = Printf.printf "%s : 0x%08lX\n" in

    Printf.printf "Magic : %s\n" header.P.magic ;
    pi32 "Text base" header.P.text_base ;
    pi32 "Data base" header.P.data_base ;
    pi32 "Init PC" header.P.init_pc ;
    pi32 "Init GP" header.P.init_gp ;
    pi32 "Text addr" header.P.text.P.addr ;
    pi32 "Text size" header.P.text.P.size ;
    pi32 "Data addr" header.P.data.P.addr ;
    pi32 "Data size" header.P.data.P.size ;
    pi32 "BSS addr" header.P.bss.P.addr ;
    pi32 "BSS size" header.P.bss.P.size ;
    pi32 "Stack addr" header.P.stack.P.addr ;
    pi32 "Stack size" header.P.stack.P.size ;

    let cl = Array.length f.P.code in
    Printf.printf "Code length : 0x%08X\n" cl

let dump_code f =
    Array.iter (fun i ->
        let instr = R3000_decoder.decode i in
        let s = R3000_decoder.to_string instr in
        print_endline s
    ) f.Psx_file.code

let parse_args () =
    let usage = "usage: disasm <-hdr|-code> <filename>" in
    let cfg = { fname = Obj.magic () ; act = Obj.magic () } in
    let opts = [
        "-hdr", Arg.Unit (fun s -> cfg.act <- dump_header),
            "dump the values in the file header" ;

        "-code", Arg.Unit (fun s -> cfg.act <- dump_code),
            "dumbly disassemble the code"
    ] in
    Arg.parse opts (fun s -> cfg.fname <- s) usage ;
    let n = Obj.magic () in
    if cfg.fname == n || cfg.act == n then (
        Arg.usage opts usage ;
        exit 1
    ) ;
    cfg

let main cfg =
    let stream = Os_fs.open_ cfg.fname in
    let psxfile = ComputerPsxFile.parse stream in
    Os_fs.close stream ;
    cfg.act psxfile

let () =
    let cfg = parse_args () in
    main cfg
