(*
 * Types used in the PSX executable format.
 *)

type segment_descriptor = {
    addr : int32 ;
    size : int32
}

type header = {
    (* Magic string : "PS-X EXE" *)
    magic : string ;

    (* Base of the text segment *)
    text_base : int32 ;

    (* Base of the data segment *)
    data_base : int32 ;

    (* First value of the PC "register" *)
    init_pc : int32 ;

    (* First value of the GP register *)
    init_gp : int32 ;

    (* Text, data, bss and stack address and size *)
    text : segment_descriptor ;
    data : segment_descriptor ;
    bss : segment_descriptor ;
    stack : segment_descriptor
}

type t = {
    header : header ;
    code : int32 array
}

module Make (FS : Filesystem.S) : sig
    val parse : FS.stream -> t
end
