(*
 * Instruction takes <= two registers and an immediate value.
 *)
module I : sig
    type args = {
        op : int ;
        rs : R3000_reg.t ;
        rt : R3000_reg.t ;
        imm : int
    }
end

(*
 * Instruction is a unconditional jump.
 *)
module J : sig
    type args = {
        op : int ;
        target : int
    }
end

(*
 * Instruction operates on registers.
 *)
module R : sig
    type args = {
        op : int ;
        rs : R3000_reg.t ;
        rt : R3000_reg.t ;
        rd : R3000_reg.t ;
        shift : int ;
        funct : int
    }
end

(*
 * Instruction is a SYSCALL.
 *)
module S : sig
    type args = {
        op : int ;
        code : int ;
        funct : int
    }
end

(*
 * Instruction branches on some register condition.
 *)
module B : sig
    type args = {
        op : int ;
        rs : R3000_reg.t ;
        typ : int ;
        imm : int
    }
end

(*
 * Include the R3000 instructions from the inline macro file.
 *)
type instr =
#define R3000_INSTR(constr, m) \
|   constr of m.args
#include "instrs.inl"
#undef R3000_INSTR

(*
 * Makes a string out of an instruction.
 *
 * Strings are not \n terminated !
 *)
val to_string : instr -> string

(*
 * Decodes an instruction from its int32 value.
 *)
val decode : int32 -> instr
