(*
 * Implements the functions to go from a R3000_regs.t to int or strings.
 *)

type t =
|   R0                                      (* Always 0 *)
|   AT                                      (* Assembler temporary *)
|   V0 | V1                                 (* Subroutine return values *)
|   A0 | A1 | A2 | A3                       (* Subroutine arguments *)
|   T0 | T1 | T2 | T3 | T4 | T5 | T6 | T7   (* Temporary registers *)
|   S0 | S1 | S2 | S3 | S4 | S5 | S6 | S7   (* Register variables *)
|   T8 | T9                                 (* Temporary registers *)
|   K0 | K1                                 (* Reserved for kernels *)
|   GP | SP | FP                            (* Global/Stack/Frame pointer *)
|   RA                                      (* Return address *)

type regdescr = {
    reg : t ;
    index : int ;
    name : string
}

let regs_by_ix, regs_by_reg, regs_by_name =
    let li = [
        { reg = R0 ; index =  0 ; name = "r0" } ;
        { reg = AT ; index =  1 ; name = "at" } ;
        { reg = V0 ; index =  2 ; name = "v0" } ;
        { reg = V1 ; index =  3 ; name = "v1" } ;
        { reg = A0 ; index =  4 ; name = "a0" } ;
        { reg = A1 ; index =  5 ; name = "a1" } ;
        { reg = A2 ; index =  6 ; name = "a2" } ;
        { reg = A3 ; index =  7 ; name = "a3" } ;
        { reg = T0 ; index =  8 ; name = "t0" } ;
        { reg = T1 ; index =  9 ; name = "t1" } ;
        { reg = T2 ; index = 10 ; name = "t2" } ;
        { reg = T3 ; index = 11 ; name = "t3" } ;
        { reg = T4 ; index = 12 ; name = "t4" } ;
        { reg = T5 ; index = 13 ; name = "t5" } ;
        { reg = T6 ; index = 14 ; name = "t6" } ;
        { reg = T7 ; index = 15 ; name = "t7" } ;
        { reg = S0 ; index = 16 ; name = "s0" } ;
        { reg = S1 ; index = 17 ; name = "s1" } ;
        { reg = S2 ; index = 18 ; name = "s2" } ;
        { reg = S3 ; index = 19 ; name = "s3" } ;
        { reg = S4 ; index = 20 ; name = "s4" } ;
        { reg = S5 ; index = 21 ; name = "s5" } ;
        { reg = S6 ; index = 22 ; name = "s6" } ;
        { reg = S7 ; index = 23 ; name = "s7" } ;
        { reg = T8 ; index = 24 ; name = "t8" } ;
        { reg = T9 ; index = 25 ; name = "t9" } ;
        { reg = K0 ; index = 26 ; name = "k0" } ;
        { reg = K1 ; index = 27 ; name = "k1" } ;
        { reg = GP ; index = 28 ; name = "gp" } ;
        { reg = SP ; index = 29 ; name = "sp" } ;
        { reg = FP ; index = 30 ; name = "fp" } ;
        { reg = RA ; index = 31 ; name = "ra" }
    ] in
    let regs_by_ix = Hashtbl.create 32 in
    let regs_by_reg = Hashtbl.create 32 in
    let regs_by_name = Hashtbl.create 32 in
    
    let add_by h f r = Hashtbl.add h (f r) r in
    List.iter (add_by regs_by_ix (fun r -> r.index)) li ;
    List.iter (add_by regs_by_reg (fun r -> r.reg)) li ;
    List.iter (add_by regs_by_name (fun r -> r.name)) li ;

    regs_by_ix, regs_by_reg, regs_by_name

let of_int n =
    (Hashtbl.find regs_by_ix n).reg

let to_int r =
    (Hashtbl.find regs_by_reg r).index

let of_string s =
    (Hashtbl.find regs_by_name s).reg

let to_string r =
    (Hashtbl.find regs_by_reg r).name
