(*
 * A common interface for filesystem modules, including :
 * - The OS filesystem
 * - The CD-ROM filesystem
 *)

type seekreq =
|   Begin
|   Absolute of int
|   Relative of int
|   End

module type S = sig
    type stream

    (* Opens a file and returns a stream to it *)
    val open_ : string -> stream

    (* Closes a stream to a file *)
    val close : stream -> unit

    (*
     * Reads max. n bytes in the file, and returns a couple (buffer, read)
     * with read = number of bytes read (could be less than the maximum
     * in case of EOF, for example).
     *)
    val read : stream -> int -> (string * int)

    (* Seeks into the file *)
    val seek : stream -> seekreq -> unit

    (* Tells where we currently are in the file *)
    val tell : stream -> int

    (* Returns true if we are at the end of the file *)
    val eof : stream -> bool
end
