/*
 * Gate between adresses and precompiled code.
 *
 * Provides a way to get the function to execute for a given address.
 */

#include "codegate.h"
#include "globals.h"
#include "interpreter.h"
#include "msg.h"

#include <stdlib.h>

/* Helpers */
static inline int in_text_seg(uint32_t addr)
{
    return (addr >= t_seg_addr) && (addr < (t_seg_addr + t_seg_size));
}

static inline int bios_call(uint32_t addr)
{
    return (addr == 0xA0 || addr == 0xB0 || addr == 0xC0);
}

instr_func psx_instr_at(uint32_t addr)
{
    /* First case : in the code segment, best case, most frequent */
    if (in_text_seg(addr))
    {
        return instrs[(addr - t_seg_addr) >> 2];
    }
    /* Second case : BIOS call */
    else if (bios_call(addr))
    {
        return psx_instr_callbios;
    }

    /* TODO: Here be programs loaded from the disk, etc. */

    /* Other cases : not yet handled ! */
    else psx_fail("unable to translate %08X to an instruction\n", addr);
}

void psx_instr_params_at(uint32_t addr, uint32_t* out)
{
    /* Same as above, except we return the arguments in an array */
    if (in_text_seg(addr))
    {
        uint32_t* i = &instrs_params[addr - t_seg_addr];

        /* Unroll the copy */
        out[0] = i[0] ; out[1] = i[1] ; out[2] = i[2] ; out[3] = i[3];
    }
    else if (bios_call(addr))
    {
        /* Let out be uninitialized, it won't be used anyway */
    }
    else psx_fail("unable to translate %08X to an instruction params set\n",
                  addr);
}
