/*
 * Defines the interpreter global state and the virtual CPU entry points.
 */

#include "address.h"
#include "globals.h"
#include "gpu.h"
#include "hardware.h"
#include "interpreter.h"
#include "memory.h"

#include <getopt.h>
#include <stdint.h>
#include <stdio.h>

static int trace_pc = 0;

static void psx_launch(void)
{
    /* Set the initial SP/GP */
    psx_set_reg(GP, init_gp);
    psx_set_reg(SP, s_seg_addr);

    /* Mainloop */
    while (pc)
    {
        if (trace_pc)
            fprintf(stderr, "[pc] %08X\n", pc);

        psx_exec_one();
    }
}

static void read_cmdline_options(int argc, char** argv)
{
    static struct option opts[] = {
        { "trace-pc",           0, &trace_pc,           1 },
        { "log-unhandled-hw",   0, &log_unhandled_hw,   1 },
        { NULL,                 0, NULL,                0 }
    };

    while (getopt_long_only(argc, argv, "", opts, NULL) != -1)
        ;
}

int main(int argc, char** argv)
{
    read_cmdline_options(argc, argv);

    psx_mem_init();
    psx_gpu_load();

    psx_launch();

    psx_gpu_unload();
    psx_mem_shutdown();

    return 0;
}
