/*
 * Functions handling read/write access to the PSX memory.
 */

#ifndef __PSX_MEMORY_H_
# define __PSX_MEMORY_H_

# include <stdint.h>

/* Size of the different memory zones */
#define RAM_SIZE 0x200000
#define PPORT_SIZE 0x10000
#define HW_SIZE 0x3000
#define BIOS_SIZE 0x40000

/* Init/shutdown */
void psx_mem_init(void);
void psx_mem_shutdown(void);

/* Normal access - checks for registers, etc. */
uint8_t psx_mem_read_8(uint32_t addr);
uint16_t psx_mem_read_16(uint32_t addr);
uint32_t psx_mem_read_32(uint32_t addr);

void psx_mem_write_8(uint32_t addr, uint8_t data);
void psx_mem_write_16(uint32_t addr, uint16_t data);
void psx_mem_write_32(uint32_t addr, uint32_t data);

/* Direct access - do not check for registers */
uint8_t psx_mem_read_8_direct(uint32_t addr);
uint16_t psx_mem_read_16_direct(uint32_t addr);
uint32_t psx_mem_read_32_direct(uint32_t addr);

void psx_mem_write_8_direct(uint32_t addr, uint8_t data);
void psx_mem_write_16_direct(uint32_t addr, uint16_t data);
void psx_mem_write_32_direct(uint32_t addr, uint32_t data);

/* Get (rw) pointers to the memory zones */
uint8_t* psx_mem_ram_ptr(uint32_t addr);

#endif
