/*
 * Extensions to the standard OCaml LLVM API.
 */

extern "C" {
#include <caml/alloc.h>
#include <caml/memory.h>
#include <caml/mlvalues.h>

/* Not a public function, but needed though */
extern CAMLprim value caml_ml_output(value chan, value buff,
                                     value start, value len);
}

#include <cstdio>
#include <cstring>
#include <llvm/Bitcode/ReaderWriter.h>
#include <llvm/Support/raw_ostream.h>
#include <llvm-c/Core.h>

class raw_ml_ostream : public llvm::raw_ostream
{
    value channel;
    uint64_t pos;

private:
    virtual void write_impl(const char* ptr, size_t size)
    {
        value str;

        str = caml_alloc_string(size + 1);
        memcpy((void*)str, (const void*)ptr, size);
        ((char*)str)[size] = 0;

        caml_ml_output(channel, str, Val_int(0), Val_int(size));

        pos += size;
    }

    virtual uint64_t current_pos() const { return pos; }
    virtual uint64_t current_pos() { return pos; }

public:
    explicit raw_ml_ostream(value chan) : channel(chan)
    {
    }
};

extern "C" CAMLprim value llvmext_print_module(LLVMModuleRef M, value chan)
{
    CAMLparam1(chan);

    raw_ml_ostream os(chan);
    llvm::unwrap(M)->print(os, NULL);

    CAMLreturn(Val_unit);
}

extern "C" CAMLprim value llvmext_write_bitcode_to_chan(
        LLVMModuleRef M, value chan)
{
    CAMLparam1(chan);

    raw_ml_ostream os(chan);
    llvm::WriteBitcodeToFile(llvm::unwrap(M), os);

    CAMLreturn(Val_unit);
}
