// DEMO4.C (c) 2001 Blade
//
// How to display a sprite, and animate it
// How to include a TIM image as an object file at compile time
// Function: Display a flapping 'robo-bird' on the screen
//           Move the bird the direction buttons are pressed
//           Exit when L2 is pressed on the pad

// again, we always need this
#include <bladeps.h>

// a routine we'll use later
void InitSprite(void);

// a define for the speed of the flapping
#define FLAP_RATE 10
u_long flap;

// this refers to our TIM image converted to an object file
// and included at compile time (see DEMO4.BAT)
extern u_char robobird[];

// a GsIMAGE to hold the TIM information
GsIMAGE tim;
// a GsSPRITE to hold the sprite information
GsSPRITE bird_sprite;

u_char primitive_buffer[2][4000];

u_long pad = 0;
u_long exit_flag = 0;

void main (void)
{
  int act;

  PadInit ();

  GsInitGraphQuick (320, 240);

  // load the TIM image and set up our sprite (robo_bird)
  InitSprite();

  while (exit_flag == 0) {

    act = GsGetActiveBuff ();

    GsSetWorkBase(&primitive_buffer[act][0]);

    GsSortClear (55, 55, 75);

    // add the sprite to the graphics queue
    GsSortSprite (&bird_sprite);

    GsSwapDispBuff ();

    GsDrawOT ((u_long *) &primitive_buffer[act][0]);

    WaitVSync();

    // flap the wings by changing the 'u' TIM offset
    flap++;
    if (flap == FLAP_RATE) {
      flap = 0;
      bird_sprite.u +=48;
      if (bird_sprite.u > 144) bird_sprite.u = 0;
      }

    pad = PadRead();

    if ((pad & PAD_L2) > 0) exit_flag++;

    // move the robo-bird as required
    // changing the 'v' TIM offset changes the direction
    if (((pad & PAD_LEFT) > 0) && (bird_sprite.x > 0)) {
      bird_sprite.x -= 2;
      bird_sprite.v = 48;
      }

    if (((pad & PAD_RIGHT) > 0) && (bird_sprite.x < 272)) {
      bird_sprite.x += 2;
      bird_sprite.v = 0;
      }

    if (((pad & PAD_UP) > 0) && (bird_sprite.y > 0))
      bird_sprite.y -= 2;

    if (((pad & PAD_DOWN) > 0) && (bird_sprite.y < 198))
      bird_sprite.y += 2;

  }

  PadStop ();
}

//=======================================================
// this routine initialises our sprite
// have a look at the TIM file in TIMtool
// it consists of 8 frames of animation (4 for each direction)
// the overall size of the TIM is 192x96
// the size of each individual frame is 48x48

void InitSprite(void)
{
  // load the TIM from memory to the frame buffer
  LoadTim((TIMHEADER *)robobird);

  // gather all the info about the TIM
  GsGetTimInfo((TIMHEADER *)robobird, &tim);

  // set the RGB values to reasonable values
  bird_sprite.r = bird_sprite.g = bird_sprite.b = 128;

  // set the initial screen position of our sprite
  bird_sprite.x = 160;
  bird_sprite.y = 120;

  // although the TIM is 192x96 we only want to show 48x48
  // sections of it
  bird_sprite.w = 48;
  bird_sprite.h = 48;

  // set the x(u) and y(v) offsets of the 48x48 sprite within
  // the TIM image
  bird_sprite.u = 0;
  bird_sprite.v = 48;

  // set the clutid and tpage for the sprite using the
  // values returned from GsTimInfo
  bird_sprite.clutid = tim.clutid;
  bird_sprite.tpage = tim.tpage;
}
